#include "StdAfx.h"
#include "GameTalkAPIGlobal.h"
#include "UtilFunction.h"
#include "MessengerActor.h"
#if defined(WIN32)
#include "WASActor.h"
//#include "RoomManager.h"
#include "ProfileImageActor.h"
#endif

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////


CGameTalkAPIGlobal::CGameTalkAPIGlobal(void)
{
	m_bGameTalkApiEnvSet = false;
	m_bGameTalkApiStarted = false;

	_LoadInternalEnv();

#ifdef __GT_UNIT_TEST__
	// UNIT TEST..
	std::string str = "NEW LINE\n\nTEST\n\nTRIM OK???\n\n\n\n\n\n";
	OutputDebugStringA(str.c_str());
	StringUtil::trim(str);
	OutputDebugStringA(str.c_str());
#endif
}


CGameTalkAPIGlobal::~CGameTalkAPIGlobal(void)
{
	messengeractormap_t::iterator it = m_mapMessengerActor.begin();
	for(; it != m_mapMessengerActor.end(); it++)
	{
		delete it->second;
	}
	m_mapMessengerActor.clear();
}


trid_t CGameTalkAPIGlobal::GetTransactionID(trid_t *pTrid)
{
	static int sg_nTrid = 1;

	if(pTrid)
	{
		if(pTrid->IsValid())
			return *pTrid;

		*pTrid = sg_nTrid++;
		return *pTrid;
	}

	return sg_nTrid++;
}


void CGameTalkAPIGlobal::SetEnvironment(const GT_API_ENVIRONMENT_DATA &sEnv)
{
	m_sEnvironmentData.Load(sEnv);

	// ȯ漳 ˻
	_VerifyEnvironmentData();

	// Ÿ
	m_sEnvironmentData.vecAuthDomains.push_back(_T("naver.com"));
	m_sEnvironmentData.vecAuthDomains.push_back(_T("hangame.com"));

	m_bGameTalkApiEnvSet = true;

	//  
	//CRITICAL_SECTION * pCS = m_apiEnvObserver.GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<_IGTAPIEnvironmentEvent>::listobserver_t observers;
	m_apiEnvObserver.GetObservers(&observers);
	CGTObserved<_IGTAPIEnvironmentEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		_IGTAPIEnvironmentEvent *pObserver = static_cast<_IGTAPIEnvironmentEvent *>(*it);
		pObserver->OnGTAPIEnvironmentEvent_Update();
	}

	//LeaveCriticalSection(pCS);
}

CMessengerActor * CGameTalkAPIGlobal::GetMessengerActor(int nUserID)
{
	messengeractormap_t::iterator it = m_mapMessengerActor.find(nUserID);
	if(it != m_mapMessengerActor.end())
	{
		return it->second;
	}

	return NULL;
}


CMessengerActor * CGameTalkAPIGlobal::CreateMessengerActor(int nUserID)
{
	CMessengerActor *pMessengerActor = new CMessengerActor;
	if (NULL == pMessengerActor)
		m_mapMessengerActor.insert(messengeractormap_t::value_type(nUserID, pMessengerActor));

	return pMessengerActor;
}

bool CGameTalkAPIGlobal::DeleteMessengerActor(int nUserID)
{
	messengeractormap_t::iterator it = m_mapMessengerActor.find(nUserID);
	if(it != m_mapMessengerActor.end())
	{
		delete it->second;
		m_mapMessengerActor.erase(it);
		return true;
	}
	return false;
}

void CGameTalkAPIGlobal::_VerifyEnvironmentData(void)
{
	int nPos = m_sEnvironmentData.strInstallPath.rfind(STR_PATH_DELIMITER);
	if(nPos != m_sEnvironmentData.strInstallPath.length() - 1)
		m_sEnvironmentData.strInstallPath += STR_PATH_DELIMITER;

	nPos = m_sEnvironmentData.strProfileImagePath.rfind(STR_PATH_DELIMITER);
	if(nPos != m_sEnvironmentData.strProfileImagePath.length() - 1)
		m_sEnvironmentData.strProfileImagePath += STR_PATH_DELIMITER;
}


void CGameTalkAPIGlobal::_LoadInternalEnv(void)
{
#if defined(WIN32)
	TCHAR szInt[32] = {'\0'};
	TCHAR szChar[256] = {'\0'};

	RegistryUtil::ReadRegistry(_T("NHN Corporation"), _T("Game Talk"), _T("GTAPI"), _T("USE_RELAY_ROOMID"), szChar, sizeof(szChar), NULL, true);
	m_sInternalEnvData.strForceRelayServerRoomIDList = szChar;

	RegistryUtil::ReadRegistry(_T("NHN Corporation"), _T("Game Talk"), _T("GTAPI"), _T("LOG_WRITE"), szInt, sizeof(szInt), NULL, true);
	m_sInternalEnvData.bTraceWrite = (_wtoi(szInt) == 1 ? true : false);

	RegistryUtil::ReadRegistry(_T("NHN Corporation"), _T("Game Talk"), _T("GTAPI"), _T("LOG_VCTRACE"), szInt, sizeof(szInt), NULL, true);
	m_sInternalEnvData.bVCTrace = (_wtoi(szInt) == 1 ? true : false);
#endif // WIN32
}
